/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.IIdLabeled;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Utils class implementing basic model related functionality for convenience.
 * 
 * @author doebber
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: 5843098F1FC2D90A0573B813E704984C
 */
public class ModelUtils {

	/**
	 * Executes model-changing {@link Runnable} on the given element's command
	 * stack. This method should only be used for runnables, which actually
	 * modify the model.
	 * 
	 * @param element
	 *            a model element, which is used to find the persisted model
	 *            project.
	 * @param command
	 *            the command to be executed.
	 */
	public static void runAsCommand(EObject element, Runnable command) {
		ICommandStackService.INSTANCE.runAsCommand(element, command);
	}

	/**
	 * Assigns unique IDs to the given target model element.
	 * 
	 * @param element
	 *            the model element, which should be prepared for insertion into
	 *            the given persisted model.
	 * @param context
	 *            a model element already contained in a persisted model.
	 * @throws NullPointerException
	 *             if the context is not contained in a persisted model.
	 */
	public static void prepareUniqueID(EObject element, EObject context) {
		// FIXME: why not use org.fortiss.tooling.kernel.utils.UniqueIDUtils.prepareIDs(EObject,
		// EObject, int)
		if(element instanceof IIdLabeled) {
			ITopLevelElement top = IPersistencyService.INSTANCE.getTopLevelElementFor(context);

			if(top == null) {
				EObject root = context;
				while(root.eContainer() != null) {
					root = root.eContainer();
				}

				// FIXME: why not start with 0
				// see also org.fortiss.tooling.kernel.internal.storage.eclipse.ModelContext:129
				int id = Integer.MIN_VALUE;
				for(TreeIterator<EObject> iter = root.eAllContents(); iter.hasNext();) {
					EObject o = iter.next();
					if(o instanceof IIdLabeled) {
						id = Math.max(id, ((IIdLabeled)o).getId());
					}
				}

				((IIdLabeled)element).setId(id + 1);
			} else {
				top.prepareIDs(element);
			}
		}
	}
}
